//---------------------------------------------------------------------
//(16x16Dot)LED Game for AVR  by Takuya Matsubara / NICO Corp. 2006-2008
// http://www.nicotak.com
//
// Marble Virtual Machine
// 
// ̕͏C֎~(Please do not delete a document from here.)
//---------------------------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "led.h"
#include "ledx.h"
#include "rand.h"
#include "sio.h"
#include "sw.h"
#include "beep.h" 


void led_printd(int x);
void led_print(char *p);

//--------------------------------------------------------------------
char fnc_abs(char a)
{
	if(a < 0){
		return(-a);
	}else{
		return(a);
	}
}

//---------------------------------------------------------------------
void led_printd(int x)
{
	unsigned char temp;
	int ketaval=1000;

	while (ketaval != 0) {
		temp = x / ketaval;
		led_putch('0'+temp);
		x -= (ketaval*temp);
		ketaval /= 10;
	}
}

//---------------------------------------------------------------------
void led_print(char *p)
{
	while(*p != 0){
		led_putch(*p++);
	}
}
//---------------------------------------------------------------
void led_disploop(char *pstr)
{
	int i=900;

	led_init();
	led_locate(0,0);
	led_print(pstr);

	while(i--){
		led_disp();
	}
}

unsigned int vmap[16];
/*
unsigned int vmap[16]={
		0b0000000000000000,
		0b0000000000000000,
		0b0000000000000000,
		0b0000000000000011,
		0b0000000011111111,
		0b0000001111111110,
		0b0000001100000000,
		0b1000000000000000,
		0b1100000000000000,
		0b1111100000000000,
		0b0111111100000000,
		0b0000011100000000,
		0b0000000000000000,
		0b0000000000000011,
		0b0000000000011111,
		0b0000000111111111
};
*/

extern unsigned int  ledvram[16*2];


char vmap_point(char x,char y)
{
	if((x<0)||(x>15))return(0);
	if((y<0)||(y>15))return(0);

	if((0x8000 >> x)& vmap[(int)y])
		return(1);

	return(0);
}

void vmap_pset(char x,char y,char c)
{
	if((x<0)||(x>15))return;
	if((y<0)||(y>15))return;

	if(c){
		vmap[(int)y] |=	(0x8000 >> x);
	}else{
		vmap[(int)y] &=	~(0x8000 >> x);
	}
}

void vmap_save(void)
{
	int adr;
	int i;
	unsigned char *pCh;

	adr = 0;
	pCh = (unsigned char *)vmap;

	for(i=0; i<sizeof(vmap); i++){
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)adr++, *pCh);
		pCh++;
	}
	eeprom_busy_wait();
}

//---------------------------------------------------------------
void vmap_load(void)
{
	int adr;
	int i;
	unsigned char zerocheck=0;
	unsigned char *pCh;

	adr = 0;
	pCh = (unsigned char *)vmap;
	for(i=0; i<sizeof(vmap); i++){
		eeprom_busy_wait();
		*pCh = eeprom_read_byte((uint8_t *)adr++);

		if((*pCh==0)||(*pCh==0xFF))
			zerocheck++;

		pCh++;
	}

	if(zerocheck >= sizeof(vmap)){
		vmap[0] = 0b0000000000000000;
		vmap[1] = 0b0000000000000000;
		vmap[2] = 0b0000000000000000;
		vmap[3] = 0b0000000000000011;
		vmap[4] = 0b0000000011111111;
		vmap[5] = 0b0000001111111110;
		vmap[6] = 0b0000001100000000;
		vmap[7] = 0b1000000000000000;
		vmap[8] = 0b1100000000000000;
		vmap[9] = 0b1111100000000000;
		vmap[10] = 0b0111111100000000;
		vmap[11] = 0b0000011100000000;
		vmap[12] = 0b0000000000000000;
		vmap[13] = 0b0000000000000011;
		vmap[14] = 0b0000000000011111;
		vmap[15] = 0b0000000111111111;
	}
}




//---------------------------------------------------------------------
void dragon(void)
{
	typedef struct {
		int x;	// XW(ۂ8{ŌvZ܂)
		int y;	// YW(ۂ8{ŌvZ܂)
		char x1;	// Xړ
		char y1;	// Yړ
		char color;
	} ST_DOT;   // DOT\

	#define DOTMAX 8	// DOTő吔
	#define CHGCNT 70 // WXV
	#define SPEEDMAX 50
	#define DOT_NONE 999

	ST_DOT dots[DOTMAX];
	ST_DOT *pdot;

	unsigned char  cnt;
	int i;
	char powerx,powery;
	int tx,ty;

	for(i=0;i<DOTMAX;i++){
		dots[i].x = DOT_NONE;
	}

	powerx=3;
	powery=3;

	while(1)
	{
		led_disp();
		cnt++;
		if(cnt >= CHGCNT) {
			cnt = 0;
			if (sw_get(SW_UP)) {			// U{^
				if(powery < 19) powery++;
			}else if (sw_get(SW_DOWN)) {	// D{^
				if(powery > 3) powery--;
			}else if (sw_get(SW_LEFT)) {	// R{^
				if(powerx < 10) powerx++;
			}else if (sw_get(SW_RIGHT)) {	// L{^
				if(powerx > 1) powerx--;
			}else if (sw_get(SW_B)) {		// B{^
				break;
			}

			for(i=0;i<16;i++){
				ledvram[i] = vmap[i];
			}
			for(i=0;i<16;i++){
				ledvram[i+16] = 0;
			}
			pdot = &dots[0];
			for(i=0;i<DOTMAX;i++){
				if(pdot->x == DOT_NONE){//continue;
					pdot->x = 15<<3;
					pdot->y = 0<<3;
					pdot->x1 = -rand_get(powerx)-1;
					pdot->y1 = rand_get(powery);
					pdot->color = rand_get(3)+1;
				}
				tx = pdot->x + pdot->x1;
				ty = pdot->y;
				if(ty >= (16<<3)){
					tx = DOT_NONE;
				}else{
					if((tx<0) || (tx>=(16<<3)) || vmap_point(tx>>3 ,ty>>3)){
						tx -= pdot->x1;	//߂荞ݖh~
						pdot->x1 = -pdot->x1;
						beep_set(40*fnc_abs(pdot->x1) + 200,1);						
					}
					ty += pdot->y1;
					if(vmap_point(tx>>3 ,ty>>3)){
						ty -= pdot->y1;	//߂荞ݖh~
						pdot->y1 = -((2*pdot->y1)/3); // oEh
						beep_set(40*fnc_abs(pdot->y1) + 200,1);						
					}else{
						pdot->y1 ++;
					}
				}
				pdot->x = tx;
				pdot->y = ty;
				led_pset (tx >>3, ty >>3, pdot->color); //hbg`

				pdot++;
			}
		}
	}
}

#define BEEPHZ_DO	262
#define BEEPHZ_RE	294
#define BEEPHZ_MI	330
#define BEEPHZ_FA	349
#define BEEPHZ_SO	392
#define BEEPHZ_RA	440
#define BEEPHZ_SI	494
#define BEEPHZ_DO2	523

//		beep_set(BEEPHZ_DO,1);

//---------------------------------------------------------------
void edit(void)
{
	#define SWCHKCNT 300   // XCb`o[v
	int x,y;
	char drawmode=0;
	char brink=0;
	int i;
	int keycnt=0;

	x=8;
	y=8;

	while(1){
		led_disp();		// LEDʍXV

		keycnt++;
		if(keycnt < SWCHKCNT) continue;
		keycnt = 0;

		if(sw_get(SW_UP)){	// U XCb`
			if(y > 0) y--;
		}else if(sw_get(SW_DOWN)){	// D XCb`
			if(y < 15) y++;
		}else if(sw_get(SW_RIGHT)){	// R XCb`
			if(x < 15) x++;
		}else if(sw_get(SW_LEFT)){	// L XCb`
			if(x > 0) x--;
		}
		if(sw_get(SW_A)){	// AXCb`
			if(drawmode==0){			
				if(vmap_point(x,y) != 0){
					drawmode = 1; //hbgOFF
				}else{
					drawmode = 2; //hbgON
				}
			}
			if(drawmode==1){
				beep_set(300,1);						
				vmap_pset(x,y,0);
			}else if(drawmode==2){
				beep_set(500,1);						
				vmap_pset(x,y,1);
			}
		}else if(sw_get(SW_B)){	// BXCb`
			break;
		}else{
			drawmode = 0;
		}
		for(i=0;i<16;i++){
			ledvram[i] = vmap[i];
		}
		for(i=0;i<16;i++){
			ledvram[i+16] = 0;
		}
		brink ^= 1;
		if(brink)
			led_pset(x,y,3);
	}
	//beep_set(600,1);						
}


//---------------------------------------------------------------------
// C
int main(void)
{
	sw_init(SW_ALL);
	beep_init();   //
	rand_init();
	sio_init();

	sei();		  
	led_disploop("START");

	while(1)
	{
		vmap_load();
		dragon();

		led_disploop("EDIT");
		edit();
		led_disploop("SAVE");
		vmap_save();
	}
}


